@@ -25,34 +25,47 @@
 
 #include "version.h"
 
-/* Set your charset here. ISO-8859-1 should be the most commonly used. */
+
+/* Set your charset here. ISO-8859-1 is default. */
 #define TARGET_CHARSET "ISO-8859-1"
 
 struct feed {
-	char * feedurl;							/* Non hashified URL */
-	char * feed;							/* Raw XML */
-	char * title;
-	char * link;
-	char * description;
-	char * lastmodified;					/* Content of header as sent by the server. */
+	char *feedurl;							/* Non hashified URL */
+	char *feed;								/* Raw XML */
+	int contentlength;
+	char *title;
+	char *link;
+	char *description;
+	char *lastmodified;						/* Content of header as sent by the server. */
 	int lasthttpstatus;
+	char *cookies;							/* Login cookies for this feed. */
+	char *authinfo;							/* HTTP authinfo string. */
+	char *servauth;							/* Server supplied authorization header. */
 	struct newsitem * items;
 	struct feed *next_ptr, *prev_ptr;
 	int problem;							/* Set if there was a problem downloading the feed. */
-	char * override;						/* Custom feed title. */
-	char * original;						/* Original feed title. */
+	char *override;							/* Custom feed title. */
+	char *original;							/* Original feed title. */
+	char *perfeedfilter;					/* Pipe feed through this program before parsing. */
+	int execurl;							/* Execurl? */
+	int smartfeed;							/* 1: new items feed. */
 	struct feedcategories * feedcategories;
 };
 
 struct newsitem {
-	struct newsitem * next_ptr, *prev_ptr;	/* Pointer to next/prev item in double linked list */
-	int readstatus;							/* 0: read, 1: unread */
+	struct newsdata *data;
+	struct newsitem *next_ptr, *prev_ptr;	/* Pointer to next/prev item in double linked list */
+};
+
+struct newsdata {
+	struct feed *parent;
+	int readstatus;							/* 0: unread, 1: read */
 	char *title;
 	char *link;
 	char *description;
 };
 
-/* Global program keybindings. Loaded from main:Config() */
+/* Global program keybindings. Loaded from setup:Config() */
 struct keybindings {
 	char next;
 	char prev;
@@ -60,6 +73,8 @@ struct keybindings {
 	char addfeed;
 	char deletefeed;
 	char markread;
+	char markunread;
+	char markallread;
 	char dfltbrowser;
 	char moveup;
 	char movedown;
@@ -78,15 +93,22 @@ struct keybindings {
 	char nofilter;
 	char help;
 	char about;
+	char perfeedfilter;
+	char andxor;
+	char home;
+	char end;
+	char enter;
+	char newheadlines;
 };
 
 /* Color definitions */
 struct color {
-	int changed;							/* Set to 1 when changing anything color related (labels) to avoid unneccessary writing of the config file. */
 	int newitems;
 	int newitemsbold;
 	int urljump;
 	int urljumpbold;
+	int feedtitle;
+	int feedtitlebold;
 };
 
 extern struct feed *first_ptr;
@@ -99,7 +121,12 @@ extern struct feed *first_ptr;
 #ifdef LOCALEPATH
 #	define _(String) gettext (String)
 #else
-#	define _(String) (String)
+#	define _(String) (String)s
+# 	define ngettext(Singular, Plural, n) (Plural)
+#endif
+
+#ifdef SUN
+#	include "os-support.h"
 #endif
 
 #endif
