@@ -1,6 +1,6 @@
 /* Snownews - A lightweight console RSS newsreader
  * 
- * Copyright 2003 Oliver Feiler <kiza@kcore.de>
+ * Copyright 2003-2004 Oliver Feiler <kiza@kcore.de>
  * http://kiza.kcore.de/software/snownews/
  *
  * interface.h
@@ -20,4 +20,9 @@
  *
  */
 
+#ifndef INTERFACE_H
+#define INTERFACE_H
+
 void UIMainInterface (void);
+
+#endif
