@@ -1,6 +1,6 @@
 /* Snownews - A lightweight console RSS newsreader
  * 
- * Copyright 2003 Oliver Feiler <kiza@kcore.de>
+ * Copyright 2003-2004 Oliver Feiler <kiza@kcore.de>
  * http://kiza.kcore.de/software/snownews/
  *
  * main.h
@@ -30,8 +30,7 @@ void Hashify (char * url);
 int UpdateFeed (struct feed * cur_ptr);
 int UpdateAllFeeds (void);
 int LoadFeed (struct feed * cur_ptr);
-int LoadAllFeeds (void);
-void SortFeeds (void);
+int LoadAllFeeds (int numfeeds);
 void WriteCache (void);
 
 #endif
