@@ -1,10 +1,13 @@
 /* Snownews - A lightweight console RSS newsreader
  * 
- * Copyright 2003 Oliver Feiler <kiza@kcore.de>
+ * Copyright 2003-2004 Oliver Feiler <kiza@kcore.de>
  * http://kiza.kcore.de/software/snownews/
  *
  * netio.c
  *
+ * Please read the file README.patching before changing any code in this file!
+ *
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
  * published by the Free Software Foundation.
@@ -25,6 +28,11 @@
 #       define _BSD_SOCKLEN_T_
 #endif
 
+/* BeOS does not define socklen_t. Using uint as suggested by port creator. */
+#ifdef __BEOS__
+#       define socklen_t unsigned int
+#endif
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,17 +52,13 @@
 #include "interface.h"
 #include "conversions.h"
 #include "ui-support.h"
-
-#ifdef SUN
-#	include "os-support.h"
-#endif
+#include "net-support.h"
 
 #define MAX_HTTP_REDIRECTS 10	/* Maximum number of redirects we will follow. */
 
 extern char *proxyname;				/* Hostname of proxyserver. */
 extern unsigned short proxyport;	/* Port on proxyserver to use. */
 extern char *useragent;
-int connectresult;
 
 /* Connect network sockets.
  *
@@ -63,12 +67,10 @@ int connectresult;
  *					3	couldn't connect
  *                  -1  aborted by user
  */
-int NetConnect (int * my_socket, char * host, int httpproto) {
+int NetConnect (int * my_socket, int * connectresult, char * host, int httpproto) {
 	int retval;
 	struct sockaddr_in address;	
 	struct hostent *remotehost;
-	char *uistring;
-	int uistringlength;
 	fd_set rfdsr;
 	fd_set rfdsw;
 	socklen_t len;
@@ -79,12 +81,6 @@ int NetConnect (int * my_socket, char * host, int httpproto) {
 	if (sscanf (host, "%[^:]:%hd", realhost, &port) != 2) {
 		port = 80;
 	}
-
-	uistringlength = strlen(_("Connecting to ")) + strlen(host) + 4;
-	uistring = malloc (uistringlength);
-	snprintf (uistring, uistringlength, _("Connecting to %s..."), host);
-	UIStatus (uistring, 0);
-	free (uistring);
 	
 	/* Create a inet stream TCP socket. */
 	*my_socket = socket (AF_INET, SOCK_STREAM, 0);
@@ -95,12 +91,6 @@ int NetConnect (int * my_socket, char * host, int httpproto) {
 	/* Set socket to nonblock mode to make it possible to interrupt the connect. */
 	fcntl(*my_socket, F_SETFL, fcntl(*my_socket, F_GETFL, 0) | O_NONBLOCK);
 	
-	/* stdin is read, socket is write, so we need read/write sets. */
-	FD_ZERO(&rfdsr);
-	FD_ZERO(&rfdsw);
-	FD_SET(0, &rfdsr);
-	FD_SET(*my_socket, &rfdsw);
-	
 	/* If proxyport is 0 we didn't execute the if http_proxy statement in main
 	   so there is no proxy. On any other value of proxyport do proxyrequests instead. */
 	if (proxyport == 0) {
@@ -118,19 +108,27 @@ int NetConnect (int * my_socket, char * host, int httpproto) {
 		memcpy (&address.sin_addr.s_addr, remotehost->h_addr_list[0], remotehost->h_length);
 			
 		/* Connect socket. */
-		connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
+		*connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
 		
 		/* Check if we're already connected.
 		   BSDs will return 0 on connect even in nonblock if connect was fast enough. */
-		if (connectresult != 0) {
+		if (*connectresult != 0) {
 			/* If errno is not EINPROGRESS, the connect went wrong. */
 			if (errno != EINPROGRESS) {
 				close (*my_socket);
 				free (realhost);
 				return 3;
 			}
-		
+			
 			while (1) {
+				/* stdin is read, socket is write, so we need read/write sets. 
+				   select() seems to destroy &rfdsw. Put the initialization
+				   inside the while() loop. */
+				FD_ZERO(&rfdsr);
+				FD_ZERO(&rfdsw);
+				FD_SET(0, &rfdsr);
+				FD_SET(*my_socket, &rfdsw);
+	
 				retval = select (*my_socket+1, &rfdsr, &rfdsw, NULL, NULL);
 				
 				if (FD_ISSET (0, &rfdsr)) {
@@ -146,10 +144,10 @@ int NetConnect (int * my_socket, char * host, int httpproto) {
 			}
 			
 			/* We get errno of connect back via getsockopt SO_ERROR (into connectresult). */
-			len = sizeof(connectresult);
-			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, &connectresult, &len);
+			len = sizeof(*connectresult);
+			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, connectresult, &len);
 			
-			if (connectresult != 0) {
+			if (*connectresult != 0) {
 				close (*my_socket);
 				free (realhost);
 				return 3;
@@ -170,7 +168,7 @@ int NetConnect (int * my_socket, char * host, int httpproto) {
 		memcpy (&address.sin_addr.s_addr, remotehost->h_addr_list[0], remotehost->h_length);
 		
 		/* Connect socket. */
-		connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
+		*connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
 		
 		/* Check if we're already connected.
 		   BSDs will return 0 on connect even in nonblock if connect was fast enough. */
@@ -182,6 +180,12 @@ int NetConnect (int * my_socket, char * host, int httpproto) {
 			}
 		
 			while (1) {
+				/* stdin is read, socket is write, so we need read/write sets. */
+				FD_ZERO(&rfdsr);
+				FD_ZERO(&rfdsw);
+				FD_SET(0, &rfdsr);
+				FD_SET(*my_socket, &rfdsw);
+				
 				retval = select (*my_socket+1, &rfdsr, &rfdsw, NULL, NULL);
 			
 				if (FD_ISSET (0, &rfdsr)) {
@@ -196,10 +200,10 @@ int NetConnect (int * my_socket, char * host, int httpproto) {
 					break;
 			}
 			
-			len = sizeof(connectresult);
-			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, &connectresult, &len);
+			len = sizeof(*connectresult);
+			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, connectresult, &len);
 			
-			if (connectresult != 0) {
+			if (*connectresult != 0) {
 				close (*my_socket);
 				free (realhost);
 				return 3;
@@ -216,9 +220,17 @@ int NetConnect (int * my_socket, char * host, int httpproto) {
 
 
 /*
- * Kiza's crufty HTTP client hack.
+ * Main network function.
+ * (Now with a useful function description *g*)
+ *
+ * This function returns the HTTP request's body (deflating gzip encoded data
+ * if needed).
+ * Updates passed feed struct with values gathered from webserver.
+ * Handles all redirection and HTTP status decoding.
+ * Returns NULL pointer if no data was received. Check httpstatus == 304,
+ * otherwise an error occured.
  */
-char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, int httpproto) {
+char * NetIO (int * my_socket, int * connectresult, char * host, char * url, struct feed * cur_ptr, char * authdata, int httpproto, int suppressoutput) {
 	char netbuf[4096];			/* Network read buffer. */
 	char *body;					/* XML body. */
 	int length;
@@ -234,26 +246,41 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 	char *newurl;				/* New document name ". */
 	char *newlocation;
 	char *tmpstring;			/* Temp pointers. */
-	char *freeme;
+	char *freeme, *freeme2;
 	char *redirecttarget;
 	fd_set rfds;
 	int retval;
 	int handled;
 	int tmphttpstatus;
 	int inflate = 0;			/* Whether feed data needs decompressed with deflate(1), gzip(2). */
-	int contentlength = 0;		/* Content-Length of server reply. */
 	int len;
 	char * inflatedbody;
-	int quirksmode = 0;
+	int quirksmode = 0;			/* IIS operation mode. */
+	int authfailed = 0;			/* Avoid repeating failed auth requests endlessly. */
 	
-	snprintf (tmp, sizeof(tmp), _("Downloading http://%s%s..."), host, url);
-	UIStatus (tmp, 0);
+	if (!suppressoutput) {
+		if (cur_ptr->title == NULL)
+			snprintf (tmp, sizeof(tmp), _("Downloading \"http://%s%s\""), host, url);
+		else
+			snprintf (tmp, sizeof(tmp), _("Downloading \"%s\""), cur_ptr->title);
 
+		UIStatus (tmp, 0);
+	}
+	
 	redirectcount = 0;
 	
 	/* Goto label to redirect reconnect. */
 	tryagain:
 	
+	/* Reconstruct digest authinfo for every request so we don't reuse
+	   the same nonce value for more than one request.
+	   This happens on superflous time on 303 redirects. */
+	if ((cur_ptr->authinfo != NULL) && (cur_ptr->servauth != NULL)) {
+		if (strstr (cur_ptr->authinfo, " Digest ") != NULL) {
+			NetSupportAuth(cur_ptr, authdata, url, cur_ptr->servauth);
+		}
+	}
+	
 	/* Open socket. */	
 	stream = fdopen (*my_socket, "r+");
 	if (stream == NULL) {
@@ -261,35 +288,66 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 		   go away if we retry.
 		   
 		   BeOS will stupidly return SUCCESS here making this code silently fail on BeOS. */
-		MainQuit (_("socket connect"), strerror(errno));
+		snprintf (tmp, sizeof(tmp), "Could not connect network socket: %s", strerror(errno));
+		UIStatus (tmp, 2);
 		return NULL;
 	}
 	
 	/* Again is proxyport == 0, non proxy mode, otherwise make proxy requests. */
 	if (proxyport == 0) {
 		/* Request URL from HTTP server. */
-		if (cur_ptr->lastmodified != NULL)
-			fprintf(stream, "GET %s HTTP/1.0\r\nAccept-Encoding: gzip,deflate\r\nUser-Agent: %s\r\nConnection: close\r\nHost: %s\r\nIf-Modified-Since: %s\r\n\r\n", url, useragent, host, cur_ptr->lastmodified);
-		else
-			fprintf(stream, "GET %s HTTP/1.0\r\nAccept-Encoding: gzip,deflate\r\nUser-Agent: %s\r\nConnection: close\r\nHost: %s\r\n\r\n", url, useragent, host);
-		
+		if (cur_ptr->lastmodified != NULL) {
+			fprintf(stream,
+					"GET %s HTTP/1.0\r\nAccept-Encoding: gzip\r\nUser-Agent: %s\r\nConnection: close\r\nHost: %s\r\nIf-Modified-Since: %s\r\n%s%s\r\n",
+					url,
+					useragent,
+					host,
+					cur_ptr->lastmodified,
+					(cur_ptr->authinfo ? cur_ptr->authinfo : ""),
+					(cur_ptr->cookies ? cur_ptr->cookies : ""));
+		} else {
+			fprintf(stream,
+					"GET %s HTTP/1.0\r\nAccept-Encoding: gzip\r\nUser-Agent: %s\r\nConnection: close\r\nHost: %s\r\n%s%s\r\n",
+					url,
+					useragent,
+					host,
+					(cur_ptr->authinfo ? cur_ptr->authinfo : ""),
+					(cur_ptr->cookies ? cur_ptr->cookies : ""));
+		}
 		fflush(stream);
 	} else {
 		/* Request URL from HTTP server. */
-		if (cur_ptr->lastmodified != NULL)
-			fprintf(stream, "GET http://%s%s HTTP/1.0\r\nAccept-Encoding: gzip,deflate\r\nUser-Agent: %s\r\nConnection: close\r\nIf-Modified-Since: %s\r\n\r\n", host, url, useragent, cur_ptr->lastmodified);
-		else
-			fprintf(stream, "GET http://%s%s HTTP/1.0\r\nAccept-Encoding: gzip,deflate\r\nUser-Agent: %s\r\nConnection: close\r\n\r\n", host, url, useragent);
-		
+		if (cur_ptr->lastmodified != NULL) {
+			fprintf(stream,
+					"GET http://%s%s HTTP/1.0\r\nAccept-Encoding: gzip\r\nUser-Agent: %s\r\nConnection: close\r\nHost: %s\r\nIf-Modified-Since: %s\r\n%s%s\r\n",
+					host,
+					url,
+					useragent,
+					host,
+					cur_ptr->lastmodified,
+					(cur_ptr->authinfo ? cur_ptr->authinfo : ""),
+					(cur_ptr->cookies ? cur_ptr->cookies : ""));
+		} else {
+			fprintf(stream,
+					"GET http://%s%s HTTP/1.0\r\nAccept-Encoding: gzip\r\nUser-Agent: %s\r\nConnection: close\r\nHost: %s\r\n%s%s\r\n",
+					host,
+					url,
+					useragent,
+					host,
+					(cur_ptr->authinfo ? cur_ptr->authinfo : ""),
+					(cur_ptr->cookies ? cur_ptr->cookies : ""));
+		}
 		fflush(stream);
 	}
 	
-	/* Use select to make the connection interuptable if it should hang. */
-	FD_ZERO(&rfds);
-	FD_SET(0, &rfds);
-	FD_SET(*my_socket, &rfds);
-	
 	while (1) {
+	/* Use select to make the connection interuptable if it should hang.
+	   select() seems to destroy &rfdsw. Put the initialization
+	   inside the while() loop. */
+		FD_ZERO(&rfds);
+		FD_SET(0, &rfds);
+		FD_SET(*my_socket, &rfds);
+	
 		retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
 		
 		if (FD_ISSET (0, &rfds)) {
@@ -302,7 +360,7 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 		if (FD_ISSET (*my_socket, &rfds))
 			break;
 	}
-
+	
 	if ((fgets (tmp, sizeof(tmp), stream)) == NULL) {
 		fclose (stream);
 		return NULL;
@@ -324,11 +382,11 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 	
 	/* If the redirectloop was run newhost and newurl were allocated.
 	   We need to free them here. */
-	if (redirectcount > 0) {
+	if ((redirectcount > 0) && (authdata == NULL)) {
 		free (host);
 		free (url);
 	}
-
+	
 	tmphttpstatus = cur_ptr->lasthttpstatus;
 	handled = 1;
 	/* Check HTTP server response and handle redirects. */
@@ -353,21 +411,23 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 				/* Give up if we reach MAX_HTTP_REDIRECTS to avoid loops. */
 				if (redirectcount > MAX_HTTP_REDIRECTS) {
 					fclose (stream);
-					UIStatus (_("Too many HTTP redirects encountered! Giving up."), 2);
+					if (!suppressoutput)
+						UIStatus (_("Too many HTTP redirects encountered! Giving up."), 2);
 					return NULL;
 				}
 				
 				while (!feof(stream)) {
 					if ((fgets (netbuf, sizeof(netbuf), stream)) == NULL) {
 						/* Something bad happened. Server sent stupid stuff. */
-						MainQuit (_("Following HTTP redirects"), strerror(errno));
+						//MainQuit (_("Following HTTP redirects"), strerror(errno));
+						return NULL;
 					}
 					/* Split netbuf into hostname and trailing url.
 					   Place hostname in *newhost and tail into *newurl.
 					   Close old connection and reconnect to server.
 					   
 					   Do not touch any of the following code! :P */
-					if (strstr (netbuf, "Location") != NULL) {
+					if (strncasecmp (netbuf, "Location", 8) == 0) {
 						//tmpstring = malloc(strlen(netbuf)+1);
 						
 						redirecttarget = strdup (netbuf);
@@ -406,9 +466,22 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 						
 						/* Change cur_ptr->feedurl on 301. */
 						if (cur_ptr->lasthttpstatus == 301) {
-							UIStatus (_("URL points to permanent redirect, updating with new location..."), 2);
+							if (!suppressoutput)
+								UIStatus (_("URL points to permanent redirect, updating with new location..."), 2);
 							free (cur_ptr->feedurl);
-							cur_ptr->feedurl = strdup (newlocation);
+							if (authdata == NULL)
+								cur_ptr->feedurl = strdup (newlocation);
+							else {
+								/* Include authdata in newly constructed URL. */
+								len = strlen(authdata) + strlen(newlocation) + 2;
+								cur_ptr->feedurl = malloc (len);
+								newurl = strdup(newlocation);
+								freeme2 = newurl;
+								strsep (&newurl, "/");
+								strsep (&newurl, "/");
+								snprintf (cur_ptr->feedurl, len, "http://%s@%s", authdata, newurl);
+								free (freeme2);
+							}
 						}
 						
 						freeme = newlocation;
@@ -425,7 +498,8 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 						   Say goodbye to the webserver in this case. In fact, we don't
 						   even say goodbye, but just drop the connection. */
 						if (newlocation == NULL) {
-							UIStatus (_("Webserver sent an invalid redirect!"), 2);
+							if (!suppressoutput)
+								UIStatus (_("Webserver sent an invalid redirect!"), 2);
 							fclose (stream);
 							return NULL;
 						}
@@ -436,19 +510,19 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 						newurl = strdup (newlocation);
 					
 						free (freeme);
-					
+						
 						/* Close connection. */	
 						fclose (stream);
-					
+						
 						/* Reconnect to server. */
-						if ((NetConnect (my_socket, newhost, httpproto)) != 0) {
+						if ((NetConnect (my_socket, connectresult, newhost, httpproto)) != 0) {
 							/* Add error handling/reporting. */
 							return NULL;
 						}
 					
 						host = newhost;
 						url = newurl;
-					
+						
 						goto tryagain;
 					}
 				}
@@ -457,10 +531,25 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 				/* Not modified received. We can close stream and return from here.
 				   Not very friendly though. :) */
 				fclose (stream);
+				/* Received good status from server, clear problem field. */
+				cur_ptr->problem = 0;
+				
+				/* This should be freed everywhere where we return
+				   and current feed uses auth. */
+				if ((redirectcount > 0) && (authdata != NULL)) {
+					free (host);
+					free (url);
+				}
 				return NULL;
+			case 401:
+				/* Authorization.
+				   Parse rest of header and rerequest URL from server using auth mechanism
+				   requested in WWW-Authenticate header field. (Basic or Digest) */
+				break;
 			case 410: /* The feed is gone. Politely remind the user to unsubscribe. */
 				snprintf (tmp, sizeof(tmp), _("The feed \"%s\" no longer exists. Please unsubscribe!"), cur_ptr->title);
-				UIStatus (tmp, 3);
+				if (!suppressoutput)
+					UIStatus (tmp, 3);
 			case 400:
 				cur_ptr->problem = 1;
 				fclose (stream);
@@ -481,14 +570,15 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 	} while(!handled);
 	
 	/* Read rest of HTTP header and parse what we need. */
-	while (!feof(stream)) {
-	
-		/* Use select to make the connection interuptable if it should hang. */
-		FD_ZERO(&rfds);
-		FD_SET(0, &rfds);
-		FD_SET(*my_socket, &rfds);
-		
+	while (!feof(stream)) {	
 		while (1) {
+			/* Use select to make the connection interuptable if it should hang.
+			   select() seems to destroy &rfdsw. Put the initialization
+	  		   inside the while() loop. */
+			FD_ZERO(&rfds);
+			FD_SET(0, &rfds);
+			FD_SET(*my_socket, &rfds);
+	   
 			retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
 			
 			if (FD_ISSET (0, &rfds)) {
@@ -506,26 +596,27 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 		   If header has longer lines than 4096 bytes something may go wrong. :) */
 		if ((fgets (netbuf, sizeof(netbuf), stream)) == NULL)
 			break;
+		
 		if (strstr (netbuf, "chunked") != NULL) {
 			/* Server sent junked encoding. Until I understand how it works
 			   and snownews uses HTTP/1.1 we must reject this answer.
 			   Chunked encoding is not defined in HTTP/1.0. */
 			chunked = 1;
 			snprintf (tmp, sizeof(tmp), _("The webserver %s sent illegal HTTP/1.0 reply! I cannot parse this."), host);
-			UIStatus (tmp, 2);
+			if (!suppressoutput)
+				UIStatus (tmp, 2);
 			cur_ptr->problem = 1;
 			fclose (stream);
 			return NULL;
 		}
 		/* Get last modified date. This is only relevant on HTTP 200. */
-		if ((strstr (netbuf, "Last-Modified") != NULL) &&
+		if ((strncasecmp (netbuf, "Last-Modified", 13) == 0) &&
 			(cur_ptr->lasthttpstatus == 200)) {
 			tmpstring = strdup(netbuf);
 			freeme = tmpstring;
 			strsep (&tmpstring, ":");
 			tmpstring++;
-			if (cur_ptr->lastmodified != NULL)
-				free(cur_ptr->lastmodified);
+			free(cur_ptr->lastmodified);
 			cur_ptr->lastmodified = malloc (strlen(tmpstring)+1);
 			strncpy (cur_ptr->lastmodified, tmpstring, strlen(tmpstring)+1);
 			if (cur_ptr->lastmodified[strlen(cur_ptr->lastmodified)-1] == '\n')
@@ -535,29 +626,95 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 			free(freeme);
 		}
 		/* Check and parse Content-Encoding header. */
-		if (strstr (netbuf, "Content-Encoding") != NULL) {
+		if (strncasecmp (netbuf, "Content-Encoding", 16) == 0) {
+			/* We do not support deflate. And probably never will.
 			if (strstr (netbuf, "deflate") != NULL)
 				inflate = 1;
+			*/
 			/* Will also catch x-gzip. */
 			if (strstr (netbuf, "gzip") != NULL)
 				inflate = 2;
+		}		
+		/* HTTP authentication
+		 *
+		 * RFC 2617 */
+		if ((strncasecmp (netbuf, "WWW-Authenticate", 16) == 0) &&
+			(cur_ptr->lasthttpstatus == 401)) {
+			if (authfailed) {
+				/* Don't repeat authrequest if it already failed before! */
+				UIStatus (_("Authentication failed!"), 3);
+				fclose (stream);
+				return NULL;
+			}
+
+			/* Remove trailing \r\n from line. */
+			if (netbuf[strlen(netbuf)-1] == '\n')
+				netbuf[strlen(netbuf)-1] = '\0';
+			if (netbuf[strlen(netbuf)-1] == '\r')
+				netbuf[strlen(netbuf)-1] = '\0';
+			
+			authfailed++;
+			
+			/* Make a copy of the WWW-Authenticate header. We use it to
+			   reconstruct a new auth reply on every loop. */
+			free (cur_ptr->servauth);
+			
+			cur_ptr->servauth = strdup (netbuf);
+			
+			/* Load authinfo into cur_ptr->authinfo. */
+			retval = NetSupportAuth(cur_ptr, authdata, url, netbuf);
+			
+			switch (retval) {
+				case 1:
+					UIStatus (_("URL does not contain authentication information!"), 2);
+					fclose (stream);
+					return NULL;
+					break;
+				case 2:
+					UIStatus (_("Could not generate authentication information!"), 2);
+					fclose (stream);
+					return NULL;
+					break;
+				case -1:
+					UIStatus (_("Unsupported authentication method requested by server!"), 2);
+					fclose (stream);
+					return NULL;
+					break;
+				default:
+					break;
+			}
+			
+			/* Close current connection and reconnect to server. */
+			fclose (stream);
+			if ((NetConnect (my_socket, connectresult, host, httpproto)) != 0) {
+				/* Add error handling/reporting. */
+				return NULL;
+			}
+
+			/* Now that we have an authinfo, repeat the current request. */
+			goto tryagain;
 		}
-		/* Check and parse Content-Length header (needed for zlib). */
-		if (strstr (netbuf, "Content-Length") != NULL) {
-	 		tmpstring = strdup (netbuf);
-			freeme = tmpstring;
-			/* Cut string at ":" sign */
-			strsep (&tmpstring, ":");
-			/* Pointer points to space after ":", advance one char */
-			tmpstring++;
-			contentlength = atoi (tmpstring);
-			free (freeme);
-	 	}
+		/* This seems to be optional and probably not worth the effort since we
+		   don't issue a lot of consecutive requests. */
+		/*if ((strncasecmp (netbuf, "Authentication-Info", 19) == 0) ||
+			(cur_ptr->lasthttpstatus == 200)) {
 		
-		if (strcmp(netbuf, "\r\n") == 0)
+		}*/
+		
+		/* HTTP RFC 2616, Section 19.3 Tolerant Applications.
+		   Accept CRLF and LF line ends in the header field. */
+		if ((strcmp(netbuf, "\r\n") == 0) || (strcmp(netbuf, "\n") == 0))
 			break;
 	}
 	
+	/* If the redirectloop was run newhost and newurl were allocated.
+	   We need to free them here.
+	   But _after_ the authentication code since it needs these values! */
+	if ((redirectcount > 0) && (authdata != NULL)) {
+		free (host);
+		free (url);
+	}
+	
 	/**********************
 	 * End of HTTP header *
 	 **********************/
@@ -571,13 +728,14 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 
 	/* Read stream until EOF and return it to parent. */
 	while (!feof(stream)) {
-	
-		/* Use select to make the connection interuptable if it should hang. */
-		FD_ZERO(&rfds);
-		FD_SET(0, &rfds);
-		FD_SET(*my_socket, &rfds);
-		
 		while (1) {
+			/* Use select to make the connection interuptable if it should hang.
+			   select() seems to destroy &rfdsw. Put the initialization
+	  		   inside the while() loop. */
+			FD_ZERO(&rfds);
+			FD_SET(0, &rfds);
+			FD_SET(*my_socket, &rfds);
+			
 			retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
 			
 			if (FD_ISSET (0, &rfds)) {
@@ -604,14 +762,14 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 	}
 	body = realloc(body, length+1);
 	body[length] = '\0';
-	
+		
 	/* Close connection. */
 	fclose (stream);
 	
 	/* If inflate==1 we need to decompress content with deflate.
 	   Probably not needed since every webserver seems to send gzip. */
 	if (inflate == 1) {
-		inflatedbody = zlib_uncompress (body, contentlength, NULL, 0);
+		inflatedbody = zlib_uncompress (body, length, NULL, 0);
 		
 		if (inflatedbody == NULL) {
 			free (body);
@@ -624,7 +782,7 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 		free (inflatedbody);
 	} else if (inflate == 2) {
 		/* gzipinflate */
-		inflatedbody = gzip_uncompress (body, contentlength, NULL);
+		inflatedbody = gzip_uncompress (body, length, NULL);
 		
 		if (inflatedbody == NULL) {
 			free (body);
@@ -636,37 +794,42 @@ char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, i
 		body = strdup (inflatedbody);
 		free (inflatedbody);
 	}
-		
+	
+	cur_ptr->contentlength = strlen (body);
+	
 	return body;
 }
 
-char * DownloadFeed (char * url, struct feed * cur_ptr) {
+/* Returns allocated string with body of webserver reply.
+   Various status info put into struct feed * cur_ptr.
+   Set suppressoutput=1 to disable ncurses calls. */
+char * DownloadFeed (char * url, struct feed * cur_ptr, int suppressoutput) {
 	int my_socket = 0;
+	int connectresult = 0;
 	int result;
 	char *host;					/* Needs to freed. */
 	char *tmphost;
 	char *freeme;
 	char *returndata;
+	char *authdata = NULL;
+	char *tmpstr;
 	char tmp[1024];
-	char httpprotostr[6];		/* http or https */
 	int httpproto = 0;			/* 0: http; 1: https */
 	
-	strncpy (httpprotostr, url, 5);
-	httpprotostr[5] = 0;
-	if (strstr (httpprotostr, "https") != NULL)
+	/* strstr will match _any_ substring. Not good, use strncasecmp with length 5! */
+	if (strncasecmp (url, "https", 5) == 0)
 		httpproto = 1;
 	else
 		httpproto = 0;
 	
-	/* hahteeteepeh://foo.bar/jaaaaaguar.html. */
-	/*              00^      0^*/
+	/* hahteeteepeh://foo.bar/jaaaaaguar.html.
+	                00^      0^
 	
-	/* Warning!
+	   Warning!
 	   This "function" will not handle the following URLs:
 	   http://foo.bar/
 	   
-	   Must add index.rdf or whatever.
-	*/
+	   Must add index.rdf or whatever. */
 	strsep (&url, "/");
 	strsep (&url, "/");
 	tmphost = url;
@@ -675,10 +838,18 @@ char * DownloadFeed (char * url, struct feed * cur_ptr) {
 		/* End of string url. Probably entered invalid stuff like
 		   http://asdf or something. Since we don't want to crash and burn
 		   return the dreaded NULL pointer. */
-		UIStatus (_("This URL doesn't look too valid to me, don't you think?"), 2);
+		if (!suppressoutput)
+			UIStatus (_("This URL doesn't look too valid to me, don't you think?"), 2);
 		return NULL;
 	}
-	/* url[0] = '\0'; */
+	
+	/* If tmphost contains an '@', extract username and pwd. */
+	if (strchr (tmphost, '@') != NULL) {
+		tmpstr = tmphost;
+		strsep (&tmphost, "@");
+		authdata = strdup (tmpstr);
+	}
+	
 	host = strdup (tmphost);
 	
 	/* netio() might change pointer of host to something else if redirect
@@ -691,38 +862,52 @@ char * DownloadFeed (char * url, struct feed * cur_ptr) {
 		url[strlen(url)-1] = '\0';
 	}
 	
-	result = NetConnect (&my_socket, host, httpproto);
+	result = NetConnect (&my_socket, &connectresult, host, httpproto);
 	
 	switch (result) {
 		case 1:
-			UIStatus (_("Couldn't create network socket!"), 2);
+			if (!suppressoutput)
+				UIStatus (_("Couldn't create network socket!"), 2);
 			cur_ptr->problem = 1;
 			free (freeme);
+			free (authdata);
 			return NULL;
 		case 2:
 			snprintf (tmp, sizeof(tmp), _("Can't resolve host %s!"), host);
-			UIStatus (tmp, 2);
+			if (!suppressoutput)
+				UIStatus (tmp, 2);
 			cur_ptr->problem = 1;
 			free (freeme);
+			free (authdata);
 			return NULL;
 		case 3:
-			snprintf (tmp, sizeof(tmp), _("Could not connect to server %s: %s"), host, strerror(connectresult));
-			UIStatus (tmp, 2);
+			/* On Solaris connect() might return -1, passing -1 to strerror()
+			   will return a NULL pointer which will segfault snprintf().
+			   This is broken IMO, but the check catches the crash. */
+			snprintf (tmp, sizeof(tmp), _("Could not connect to server %s: %s"), host,
+				(strerror(connectresult) ? strerror(connectresult) : "(null)"));
+			if (!suppressoutput)
+				UIStatus (tmp, 2);
 			cur_ptr->problem = 1;
 			free (freeme);
+			free (authdata);
 			return NULL;
 		case -1:
-			UIStatus (_("Aborted."), 2);
+			if (!suppressoutput)
+				UIStatus (_("Aborted."), 2);
 			free (freeme);
+			free (authdata);
 			return NULL;
 		default:
 			break;
 	}
 	
-	returndata = NetIO (&my_socket, host, url, cur_ptr, httpproto);
+	returndata = NetIO (&my_socket, &connectresult, host, url, cur_ptr, authdata, httpproto, suppressoutput);
 	
 	/* url will be freed in the calling function. */
 	free (freeme);		/* This is *host. */
 	
+	free (authdata);
+	
 	return returndata;
 }
