@@ -1,6 +1,6 @@
 /* Snownews - A lightweight console RSS newsreader
  * 
- * Copyright 2003 Oliver Feiler <kiza@kcore.de>
+ * Copyright 2003-2004 Oliver Feiler <kiza@kcore.de>
  * http://kiza.kcore.de/software/snownews/
  *
  * netio.h
@@ -20,4 +20,9 @@
  *
  */
 
-char * DownloadFeed (char * url, struct feed * cur_ptr);
+#ifndef NETIO_H
+#define NETIO_H
+
+char * DownloadFeed (char * url, struct feed * cur_ptr, int suppressoutput);
+
+#endif
