@@ -1,7 +1,7 @@
 /* Snownews - A lightweight console RSS newsreader
  * 
- * Copyright 2003 Rene Puls <rpuls@gmx.net> and
- *                Oliver Feiler <kiza@kcore.de>
+ * Copyright 2003-2004 Rene Puls <rpuls@gmx.net> and
+ *                     Oliver Feiler <kiza@kcore.de>
  * Written for Snownews.
  *
  * http://kiza.kcore.de/software/snownews/
@@ -9,6 +9,9 @@
  *
  * xmlparse.c
  *
+ * Please read the file README.patching before changing any code in this file!
+ *
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
  * published by the Free Software Foundation.
@@ -29,6 +32,7 @@
 
 #include "config.h"
 #include "xmlparse.h"
+#include "conversions.h"
 
 int saverestore;
 struct newsitem *copy;
@@ -37,35 +41,27 @@ struct newsitem *firstcopy;
 /* Wird whrend des Parsens aufgerufen, falls wir auf ein <channel> Element
    treffen. Die Funktion gibt ein neues Struct fr den Newsfeed zurck. */
 
-void parse_rdf10_channel(struct feed * feed, xmlDocPtr doc, xmlNodePtr node) 
-{
+void parse_rdf10_channel(struct feed * feed, xmlDocPtr doc, xmlNodePtr node) {
 	xmlNodePtr cur;
 	
 	/* Free everything before we write to it again. */
-	if (feed->title != NULL)
-		free (feed->title);
-	if (feed->link != NULL)
-		free (feed->link);
-	if (feed->description != NULL)
-		free (feed->description);
+	free (feed->title);
+	free (feed->link);
+	free (feed->description);
 		
 	if (feed->items != NULL) {
 		while (feed->items->next_ptr != NULL) {
 			feed->items = feed->items->next_ptr;
-			if (feed->items->prev_ptr->title != NULL)
-				free (feed->items->prev_ptr->title);
-			if (feed->items->prev_ptr->link != NULL)
-				free (feed->items->prev_ptr->link);
-			if (feed->items->prev_ptr->description)
-				free (feed->items->prev_ptr->description);
+			free (feed->items->prev_ptr->data->title);
+			free (feed->items->prev_ptr->data->link);
+			free (feed->items->prev_ptr->data->description);
+			free (feed->items->prev_ptr->data);
 			free (feed->items->prev_ptr);
 		}
-		if (feed->items->title != NULL)
-			free (feed->items->title);
-		if (feed->items->link != NULL)
-			free (feed->items->link);
-		if (feed->items->description != NULL)
-			free (feed->items->description);
+		free (feed->items->data->title);
+		free (feed->items->data->link);
+		free (feed->items->data->description);
+		free (feed->items->data);
 		free (feed->items);
 	}
 	
@@ -81,22 +77,29 @@ void parse_rdf10_channel(struct feed * feed, xmlDocPtr doc, xmlNodePtr node)
 			continue;
 		if (xmlStrcmp(cur->name, "title") == 0) {
 			feed->title = xmlNodeListGetString(doc, cur->children, 1);
+			CleanupString (feed->title, 1);
 			/* Remove trailing newline */
 			if (feed->title != NULL) {
-				if (feed->title[strlen(feed->title)-1] == '\n')
-					feed->title[strlen(feed->title)-1] = '\0';
+				if (strlen(feed->title) > 1) {
+					if (feed->title[strlen(feed->title)-1] == '\n')
+						feed->title[strlen(feed->title)-1] = '\0';
+				}
 			}
 		}
 		else if (xmlStrcmp(cur->name, "link") == 0) {
 			feed->link = xmlNodeListGetString(doc, cur->children, 1);
 			/* Remove trailing newline */
 			if (feed->link != NULL) {
-				if (feed->link[strlen(feed->link)-1] == '\n')
-					feed->link[strlen(feed->link)-1] = '\0';
+				if (strlen(feed->link) > 1) {
+					if (feed->link[strlen(feed->link)-1] == '\n')
+						feed->link[strlen(feed->link)-1] = '\0';
+				}
 			}
 		}
-		else if (xmlStrcmp(cur->name, "description") == 0)
+		else if (xmlStrcmp(cur->name, "description") == 0) {
 			feed->description = xmlNodeListGetString(doc, cur->children, 1);
+			CleanupString (feed->description, 0);
+		}
 	}
 }
 
@@ -106,30 +109,23 @@ void parse_rdf20_channel(struct feed * feed, xmlDocPtr doc, xmlNodePtr node)
 	xmlNodePtr cur;
 	
 	/* Free everything before we write to it again. */
-	if (feed->title != NULL)
-		free (feed->title);
-	if (feed->link != NULL)
-		free (feed->link);
-	if (feed->description != NULL)
-		free (feed->description);
+	free (feed->title);
+	free (feed->link);
+	free (feed->description);
 		
 	if (feed->items != NULL) {
 		while (feed->items->next_ptr != NULL) {
 			feed->items = feed->items->next_ptr;
-			if (feed->items->prev_ptr->title != NULL)
-				free (feed->items->prev_ptr->title);
-			if (feed->items->prev_ptr->link != NULL)
-				free (feed->items->prev_ptr->link);
-			if (feed->items->prev_ptr->description)
-				free (feed->items->prev_ptr->description);
+			free (feed->items->prev_ptr->data->title);
+			free (feed->items->prev_ptr->data->link);
+			free (feed->items->prev_ptr->data->description);
+			free (feed->items->prev_ptr->data);
 			free (feed->items->prev_ptr);
 		}
-		if (feed->items->title != NULL)
-			free (feed->items->title);
-		if (feed->items->link != NULL)
-			free (feed->items->link);
-		if (feed->items->description != NULL)
-			free (feed->items->description);
+		free (feed->items->data->title);
+		free (feed->items->data->link);
+		free (feed->items->data->description);
+		free (feed->items->data);
 		free (feed->items);
 	}
 	
@@ -145,23 +141,29 @@ void parse_rdf20_channel(struct feed * feed, xmlDocPtr doc, xmlNodePtr node)
 			continue;
 		if (xmlStrcmp(cur->name, "title") == 0) {
 			feed->title = xmlNodeListGetString(doc, cur->children, 1);
+			CleanupString (feed->title, 1);
 			/* Remove trailing newline */
 			if (feed->title != NULL) {
-				if (feed->title[strlen(feed->title)-1] == '\n')
-					feed->title[strlen(feed->title)-1] = '\0';
+				if (strlen(feed->title) > 1) {
+					if (feed->title[strlen(feed->title)-1] == '\n')
+						feed->title[strlen(feed->title)-1] = '\0';
+				}
 			}
 		}
 		else if (xmlStrcmp(cur->name, "link") == 0) {
 			feed->link = xmlNodeListGetString(doc, cur->children, 1);
 			/* Remove trailing newline */
 			if (feed->link != NULL) {
-				if (feed->link[strlen(feed->link)-1] == '\n')
-					feed->link[strlen(feed->link)-1] = '\0';
+				if (strlen(feed->link) > 1) {
+					if (feed->link[strlen(feed->link)-1] == '\n')
+						feed->link[strlen(feed->link)-1] = '\0';
+				}
 			}
 		}
-		else if (xmlStrcmp(cur->name, "description") == 0)
+		else if (xmlStrcmp(cur->name, "description") == 0) {
 			feed->description = xmlNodeListGetString(doc, cur->children, 1);
-		else if (xmlStrcmp(cur->name, "item") == 0) {
+			CleanupString (feed->description, 0);
+		} else if (xmlStrcmp(cur->name, "item") == 0) {
 			parse_rdf10_item(feed, doc, cur->children);
 		}
 	}
@@ -182,12 +184,13 @@ void parse_rdf10_item(struct feed *feed, xmlDocPtr doc, xmlNodePtr node)
 	
 	/* Speicher fr ein neues Newsitem reservieren */
 	item = malloc(sizeof (struct newsitem));
+	item->data = malloc (sizeof (struct newsdata));
 	
-	item->title = NULL;
-	item->link = NULL;
-	item->description = NULL;
-	
-	item->readstatus = 0;
+	item->data->title = NULL;
+	item->data->link = NULL;
+	item->data->description = NULL;
+	item->data->readstatus = 0;
+	item->data->parent = feed;
 		
 	/* Alle Tags im <item> Tag durchgehen und die Informationen extrahieren.
 	   Selbe Vorgehensweise wie in der parse_channel() Funktion */
@@ -195,32 +198,38 @@ void parse_rdf10_item(struct feed *feed, xmlDocPtr doc, xmlNodePtr node)
 		if (cur->type != XML_ELEMENT_NODE)
 			continue;
 		if (xmlStrcmp(cur->name, "title") == 0) {
-			item->title = xmlNodeListGetString(doc, cur->children, 1);
+			item->data->title = xmlNodeListGetString(doc, cur->children, 1);
+			CleanupString (item->data->title, 1);
 			/* Remove trailing newline */
-			if (item->title != NULL) {
-				if (item->title[strlen(item->title)-1] == '\n')
-					item->title[strlen(item->title)-1] = '\0';
+			if (item->data->title != NULL) {
+				if (strlen(item->data->title) > 1) {
+					if (item->data->title[strlen(item->data->title)-1] == '\n')
+						item->data->title[strlen(item->data->title)-1] = '\0';
+				}
 			}
 		}
 		else if (xmlStrcmp(cur->name, "link") == 0) {
-			item->link = xmlNodeListGetString(doc, cur->children, 1);
-			if (item->link == NULL) {
+			item->data->link = xmlNodeListGetString(doc, cur->children, 1);
+			if (item->data->link == NULL) {
 				if (xmlStrcmp(cur->name, "guid") == 0)
-					item->link = xmlNodeListGetString(doc, cur->children, 1);
+					item->data->link = xmlNodeListGetString(doc, cur->children, 1);
 			}
 			/* Remove trailing newline */
-			if (item->link != NULL) {
-				if (item->link[strlen(item->link)-1] == '\n')
-					item->link[strlen(item->link)-1] = '\0';
+			if (item->data->link != NULL) {
+				if (strlen(item->data->link) > 1) {
+					if (item->data->link[strlen(item->data->link)-1] == '\n')
+						item->data->link[strlen(item->data->link)-1] = '\0';
+				}
 			}
 		}
 		else if (xmlStrcmp(cur->name, "description") == 0) {
-			item->description = xmlNodeListGetString(doc, cur->children, 1);
+			item->data->description = xmlNodeListGetString(doc, cur->children, 1);
+			CleanupString (item->data->description, 0);
 		}
 		else if (xmlStrcmp(cur->name, "readstatus") == 0) {
 			/* Will cause memory leak otherwise, xmlNodeListGetString must be freed. */
 			readstatusstring = xmlNodeListGetString(doc, cur->children, 1);
-			item->readstatus = atoi (readstatusstring);
+			item->data->readstatus = atoi (readstatusstring);
 			xmlFree (readstatusstring);
 		}
 	}
@@ -228,9 +237,9 @@ void parse_rdf10_item(struct feed *feed, xmlDocPtr doc, xmlNodePtr node)
 	/* If saverestore == 1, restore readstatus. */
 	if (saverestore == 1) {
 		for (current = firstcopy; current != NULL; current = current->next_ptr) {
-			if ((current->link != NULL) && (item->link != NULL)) {
-				if (strcmp(item->link, current->link) == 0)
-					item->readstatus = current->readstatus;
+			if ((current->data->link != NULL) && (item->data->link != NULL)) {
+				if (strcmp(item->data->link, current->data->link) == 0)
+					item->data->readstatus = current->data->readstatus;
 			}
 		}
 	}
@@ -266,13 +275,14 @@ int DeXML (struct feed * cur_ptr) {
 		/* Copy current newsitem struct. */	
 		for (cur_item = cur_ptr->items; cur_item != NULL; cur_item = cur_item->next_ptr) {
 			copy = malloc (sizeof(struct newsitem));
-			copy->title = NULL;
-			copy->description = NULL;
-			copy->readstatus = cur_item->readstatus;
-			if (cur_item->link != NULL)
-				copy->link = strdup (cur_item->link);
+			copy->data = malloc (sizeof (struct newsdata));
+			copy->data->title = NULL;
+			copy->data->description = NULL;
+			copy->data->readstatus = cur_item->data->readstatus;
+			if (cur_item->data->link != NULL)
+				copy->data->link = strdup (cur_item->data->link);
 			else
-				copy->link = NULL;
+				copy->data->link = NULL;
 			
 			copy->next_ptr = NULL;
 			if (firstcopy == NULL) {
@@ -287,9 +297,6 @@ int DeXML (struct feed * cur_ptr) {
 		}
 	}
 	
-	/* Datei lesen */
-	/* doc = xmlParseMemory(cur_ptr->feed, strlen(cur_ptr->feed)); */
-	
 	/* xmlRecoverMemory:
 	   parse an XML in-memory document and build a tree.
        In case the document is not Well Formed, a tree is built anyway. */
@@ -345,10 +352,12 @@ int DeXML (struct feed * cur_ptr) {
 		/* free struct newsitem *copy. */
 		while (firstcopy->next_ptr != NULL) {
 			firstcopy = firstcopy->next_ptr;
-			free (firstcopy->prev_ptr->link);
+			free (firstcopy->prev_ptr->data->link);
+			free (firstcopy->prev_ptr->data);
 			free (firstcopy->prev_ptr);
 		}
-		free (firstcopy->link);
+		free (firstcopy->data->link);
+		free (firstcopy->data);
 		free (firstcopy);
 	}
 	
@@ -362,8 +371,7 @@ int DeXML (struct feed * cur_ptr) {
 	
 	/* Restore custom title. */
 	if (cur_ptr->override != NULL) {
-		if (cur_ptr->title != NULL)
-			free (cur_ptr->title);
+		free (cur_ptr->title);
 		cur_ptr->title = strdup (cur_ptr->override);
 	}
 		
